#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp     = 0;
my $pUser     = 'root';
my $pPassword = '';
my $pHost     = 'localhost';
my $lDefaultPort   = '3306';
my $lDefaultSocket = '/tmp/mysql.sock';

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags parameters

DESCRIPTION

  Nagios plugin to check if a MySQL database is up and running...

  For security reasons use a user with as little privileges as possible. For
  example a user called check_db:

  GRANT USAGE ON *.* TO 'check_db'\@'monitoring_server' IDENTIFIED BY 'check_db';

FLAGS

  --help, -?          Display this help and exit.
  --host=name, -h     Host where database to check is located (default $pHost).
  --password=name, -p Password of user $pUser to use when connecting to server
                      $pHost (default '$pPassword').
  --port=#, -P        Port number where database listens to (default $lDefaultPort).

  --socket=name, -S   Socket file to use for connection (default $lDefaultSocket).
  --user=name, -u     Check user for connecting to the database (default $pUser).

PARAMETERS

  none

EXAMPLE

  $lMyName --user=check_db --password=check_db --host=mysql_host --port=3308

EOF
}

my ($pPort, $pSocket);

my $rc = GetOptions( 'help|?' => \$pHelp
                   , 'user|u=s' => \$pUser
                   , 'host|h=s' => \$pHost
                   , 'password|p=s' => \$pPassword
                   , 'port|P=i' => \$pPort
                   , 'socket|S=s' => \$pSocket
                   );

# Use either port or socket
# Rule is: prefer socket

my $lConnectionMethod = 'socket';

if ( ! defined($pPort) )
{
  $pPort = $lDefaultPort;
}
if ( ! defined($pSocket) )
{
  $pSocket = $lDefaultSocket;
}

# Port AND Socket are given. That is one too much!
if ( ($pSocket ne $lDefaultSocket) && ($pPort ne $lDefaultPort) )
{
  print("Port AND Socket are overspecifed.");
  exit($lError{'Warning'});
}

if ( $pPort ne $lDefaultPort )
{
  $lConnectionMethod = 'port';
}

if ( $pHelp ) {
    &Usage();
    exit(0);
}

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort, Socket = $pSocket");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my $cmd;
if ( $lConnectionMethod eq 'socket' )
{
  $cmd = "mysqladmin --user=$pUser --password=\'$pPassword\' --socket=$pSocket ping 2>/dev/null";
}
else
{
  $cmd = "mysqladmin --user=$pUser --host=$pHost --port=$pPort --password=\'$pPassword\' --protocol=tcp ping 2>/dev/null";
}

`$cmd`;
$rc = $?;

if ( $rc ) {
  print("ERROR. mysqladmin ping failed to connect.");
  exit($lError{'Critical'});
}

my ($dbh, $sql, $sth);

if ( $lConnectionMethod eq 'socket' )
{
  $dbh = DBI->connect("DBI:mysql::mysql_socket=$pSocket"
                    , $pUser, $pPassword
                    , { RaiseError => 0
                      , PrintError => 0
                      , AutoCommit => 0
                      }
                    );
  }
else
{
  $dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                    , $pUser, $pPassword
                    , { RaiseError => 0
                      , PrintError => 0
                      , AutoCommit => 0
                      }
                    );
}

if ( DBI::err ) {

  if ( DBI::errstr =~ m/Can't connect to/ ) {
    print("Error during connection: " . DBI::errstr);
    exit($lError{'Critical'});
  }

  if ( DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database: " . DBI::errstr);
    exit($lError{'Warning'});
  }

  print("Error during connection: " . DBI::errstr);
  exit($lError{'Critical'});
}

$sql = "
  SELECT USER()
";

$sth = $dbh->prepare( $sql );
if ( DBI::err ) {
  print("Error in preparing: " . DBI::errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my $lUser;
$sth->bind_columns(undef, \$lUser);
if ( $sth->err ) {
  print("Error in binding: " . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
if ( $sth->err ) {
  print("Error in fetchting:" . $sth->err);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->finish;
$dbh->disconnect;

print "Database seems up and running...";
exit($lError{'OK'});
