#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp = 0;
chomp($_ = `hostname`);
my $pErrorLog = '/var/mysql/data/' . $_  . '.err';
my $pDatabaseName = 'defaultDB';
my $pConfigurationFile = '/tmp/check_errorlog_mysql.conf';

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags parameters

DESCRIPTION

  Nagios plugin to check the MySQL error log...

FLAGS

  --help, -?          Display this help and exit.
  --error-log, -e     Name of the error log file (default $pErrorLog).
  --instance, -i      Name of the MySQL instance. This is important if more than
                      one instance is running on the same host (default $pDatabaseName).
  --configuration, -c Location of the configuration file where last read position is
                      stored (default $pConfigurationFile).

PARAMETERS

  check_errorlog_mysql.pl --error-log=/home/mysql/data/mysql-5.5.14/laptop.err

EXAMPLE

  $lMyName --instance=$pDatabaseName, --error-log=$pErrorLog --configuration=$pConfigurationFile

EOF
}

my $rc = GetOptions(
    'error-log|e=s' => \$pErrorLog
  , 'database|d=s' => \$pDatabaseName
  , 'configuration|c=s' => \$pConfigurationFile
  , 'help|?' => \$pHelp
);

if ( $pHelp ) {
    &Usage();
    exit(0);

}

if ( ! $rc ) {
  print("Error in parameters.");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------
my $lConfigurationFileTmp = "$pConfigurationFile.tmp";

my @lFilterConfiguration = (
    '^Version: \'.*\'  socket: \'.*\'  port: \d*\s*.*$'
  , '^Number of processes running now: \d*$'
  , '^handlersocket: initialized$'
  , '^.*: ready for connections.?$'
  , '^InnoDB: a new database to be created!$'
  , '^InnoDB: Creating foreign key constraint system tables$'
  , '^InnoDB: Database physically writes the file full: wait...$'
  , '^InnoDB: Doublewrite buffer created$'
  , '^InnoDB: Doublewrite buffer not found: creating new$'
  , '^InnoDB: Foreign key constraint system tables created$'
  , '^InnoDB: Setting log file .* size to \d* MB$'
  , '^InnoDB: The first specified data file .* did not exist:$'
  , '^InnoDB: Reading tablespace information from the .ibd files...$'
  , '^InnoDB: buffer...$'
  , '^InnoDB: log sequence number \d* \d*.$'
  , '^InnoDB: Progress in percents:(\s\d*)*$'
  , '^InnoDB: Apply batch completed$'
  , '^InnoDB: The InnoDB memory heap is disabled$'
  , '^InnoDB: Mutexes and rw_locks use GCC atomic builtins\.?$'
  , '^InnoDB: Mutexes and rw_locks use InnoDB\'s own implementation$'
  , '^InnoDB: Compressed tables use zlib \d+.\d+.\d+$'
  , '^InnoDB: Compressed tables use zlib \d+.\d+.\d+.\d+$'
  , '^InnoDB: \d+ rollback segment\(s\) active.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave SQL thread initialized, starting replication in log \'.*\' at position \d*, relay log \'.*\' position: \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave SQL thread exiting, replication stopped in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread killed while reading event$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread exiting, read up to log \'.*\', position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave I/O thread: connected to master \'.*\',\s*replication started in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Slave: connected to master \'.*\',replication resumed in log \'.*\' at position \d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Shutting down plugin \'\w+\'$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Giving client threads a chance to die gracefully$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Shutting down slave threads$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Forcefully disconnecting remaining clients$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Binlog end$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: ready for connections.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: Shutdown complete$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] .*: Normal shutdown$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Plugin \'FEDERATED\' is disabled.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Event Scheduler: Loaded \d+ events$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Event Scheduler: Purging the queue. \d+ events$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase XT \(PBXT\) Engine .* loaded...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase XT Engine shutdown...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase BLOB Management Sysytem \(PBMS\) Daemon \d+.\d+.\d+ loaded...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase BLOB Management Sysytem \(PBMS\) Daemon Setting Server ID: \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase BLOB Management Sysytem \(PBMS\) Daemon listening on port \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] PrimeBase Media Stream Daemon no longer published$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Barry Leslie, PrimeBase Technologies GmbH, http://www.primebase.org$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Paul McCullagh, PrimeBase Technologies GmbH, http://www.primebase.org$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Starting Cluster Binlog Thread$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Stopping Cluster Binlog$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Stopping Cluster Utility thread$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Restarting Cluster Binlog$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] Start binlog_dump to slave_server\(\d+\), pos\(bin-log.\d+, \d+\)$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB: Creating mysql.ndb_\w+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB: Flushing mysql.ndb_\w+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB: NodeID is \d+, management server \'localhost:\d+\'$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB\[\d+\]: NodeID: \d+, all storage nodes connected$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: starting log at epoch \d+/\d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: Node: \d+, down, Subscriber bitmask \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: cluster failure for ./mysql/ndb_\w+ at epoch \d+/\d+.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: ndb tables writable$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: ndb tables initially read only on reconnect.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: Ndb tables initially read only.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: CREATE TABLE Event: REPL\$mysql/ndb_\w+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*\[Note\] NDB Binlog: logging \./mysql/ndb_\w+ \(UPDATED,USE_WRITE\)$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Log file .* did not exist: new to be created$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Setting file .* size to \d* MB$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Shutdown completed; log sequence number \d*\s*\d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Started; log sequence number \d*\s*\d*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Started$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting an apply batch of log records to the database...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting log scan based on checkpoint at$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Starting shutdown...$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Shutdown completed$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Initializing buffer pool, size = \d+.\d+[GM]$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Completed initialization of buffer pool$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: highest supported file format is Barracuda.$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: The InnoDB memory heap is disabled$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Mutexes and rw_locks use GCC atomic builtins$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Compressed tables use zlib \d+.\d+.\d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Compressed tables use zlib \d+.\d+.\d+.\d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Waiting for the background threads to start$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: \d+.\d+.\d+ started; log sequence number \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB: Using Linux native AIO$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB Plugin \d+.\d+.\d+ started; log sequence number \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*InnoDB Plugin \d+.\d+.\d+-\d+ started; log sequence number \d+$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*mysqld (ended|started)$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*mysqld_safe Starting mysqld daemon with databases from .*$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*mysqld_safe mysqld from pid file .* ended$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*.*: Shutdown Complete$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*.*: Normal shutdown$'
  , '^\d{6}\s*\d{1,2}:\d{2}:\d{2}\s*Percona XtraDB \(http://www.percona.com\) .* started; log sequence number \d+$'
  , '^\d{4}-\d{2}-\d{2}\s*\d{1,2}:\d{2}:\d{2}\s*\[NdbApi\] INFO     -- Flushing incomplete GCI:s \< \d+/\d+$'
);

# Read last position form configuration file
# ------------------------------------------
my $lOldTs = 0;

if ( -r $pConfigurationFile ) {
  open(CONF, '<' . $pConfigurationFile);
  while ( <CONF> ) {
    chomp;
    if ( m/^$pDatabaseName:(\d*)$/ ) {
      $lOldTs = $1;
      last;
    }
  }
  close(CONF);
}
else {
  `touch $pConfigurationFile`;
}

# Parse the error log
# -------------------

my $lFilter;
my $lTs;
my $f;
my $lNewTs = 0;
my $lErrorCounter = 0;
my $lWarningCounter = 0;

if ( ! -r $pErrorLog ) {
  print("No mysql error log found on $pErrorLog. Please check configuration and verify that you have read access!");
  exit($lError{'Warning'});
}

open(LOG, '<' . $pErrorLog);
my $lInTime = 0;
while ( <LOG> ) {

  # skip empty lines
  if ( m/^$/ ) {
    next;
  }

  if ( (! $lInTime) and m/^(\d{6}\s*\d{1,2}:\d{2}:\d{2})/ ) {
     $lTs = $1;
     $lTs =~  s/[ :]//g;
     # potential loss of records within the same second!!!
     if ( $lTs <= $lOldTs ) {
       next;
     }
     else {
       $lInTime = 1;
     }
  }

  if ($lInTime) {

    $f = 0;
    foreach $lFilter (@lFilterConfiguration) {

      if ( m/$lFilter/ ) {
        $f++;
        last;
      }
    }
    if ( $f ) {
      next;
    }

    print $_;
    if ( m/error/i ) {
      $lErrorCounter++;
    }
    elsif ( m/warning/i ) {
      $lWarningCounter++;
    }

    # Time stamps can occur like this:
    # 110523 11:08:38
    # 110803  2:20:59

    if ( m/^(\d{6})\s*(\d{1,2}):(\d{2}:\d{2})/ ) {
      $lNewTs = sprintf("%s%02d%s", $1, $2, $3);
      $lNewTs =~ s/[ :]//g;
    }
  }
}
close(LOG);

$lNewTs =~ s/^0*//;

if ( $lNewTs eq '' ) {
  $lNewTs = $lOldTs;
}

# Set last position
# -----------------

open(CONF, '<' . $pConfigurationFile);
open(TMP, '>' . $lConfigurationFileTmp);
while ( <CONF> ) {
  if ( ! m/^$pDatabaseName:(\d*)$/ ) {
    print(TMP $_);
  }
}
print(TMP "$pDatabaseName:$lNewTs\n");
close(TMP);
close(CONF);
`mv $lConfigurationFileTmp $pConfigurationFile`;
$rc = $?;

if ( $rc ) {
  print("Cannot move temporary file from $lConfigurationFileTmp to $pConfigurationFile");
  exit($lError{'Warning'});
}

if ( $lErrorCounter gt 0 ) {
  # print("Some new errors found!");
  exit($lError{'Critical'});
}
elsif ( $lWarningCounter ) {
  exit($lError{'Warning'});
}

print("No errors or warnings found.");
exit($lError{'OK'});
