#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser oli.sennhauser@fromdual.com.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp     = 0;
my $pUser     = 'root';
my $pPassword = '';
my $pHost     = 'localhost';
my $pNodes    = 3;
my $lDefaultPort   = '3306';
my $lDefaultSocket = '/tmp/mysql.sock';

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags parameters

DESCRIPTION

  Nagios plugin to check if a Galera Cluster for MySQL is up and running...

  For security reasons use a user with as little privileges as possible. For
  example a user called check_db:

  GRANT USAGE ON *.* TO 'check_db'\@'monitoring_server' IDENTIFIED BY 'check_db';

FLAGS

  --help, -?          Display this help and exit.
  --host=name, -h     Host where database to check is located (default $pHost).
  --nodes=n           Number of expected nodes (default $pNodes).
  --password=name, -p Password of user $pUser to use when connecting to server
                      $pHost (default '$pPassword').
  --port=#, -P        Port number where database listens to (default $lDefaultPort).

  --socket=name, -S   Socket file to use for connection (default $lDefaultSocket).
  --user=name, -u     Check user for connecting to the database (default $pUser).

PARAMETERS

  none

EXAMPLE

  $lMyName --user=$pUser --password=secret --host=$pHost --port=$lDefaultPort --nodes=$pNodes

EOF
}

my ($pPort, $pSocket);

my $rc = GetOptions( 'help|?' => \$pHelp
                   , 'user|u=s' => \$pUser
                   , 'host|h=s' => \$pHost
                   , 'nodes|n=i' => \$pNodes
                   , 'password|p=s' => \$pPassword
                   , 'port|P=i' => \$pPort
                   , 'socket|S=s' => \$pSocket
                   );

# Use either port or socket
# Rule is: prefer socket

my $lConnectionMethod = 'socket';

if ( ! defined($pPort) ) {
  $pPort = $lDefaultPort;
}
if ( ! defined($pSocket) ) {
  $pSocket = $lDefaultSocket;
}

# Port AND Socket are given. That is one too much!
if ( ($pSocket ne $lDefaultSocket) && ($pPort ne $lDefaultPort) ) {
  print("Port AND Socket are overspecifed.");
  exit($lError{'Warning'});
}

# Nodes not specfied or values out of range (<2 > 64)

if ( ($pNodes < 2) || ($pNodes > 128) ) {
  print "Number for Galera Cluster nodes is out of expected range (2...128): $pNodes\n";
  exit($lError{'Warning'});
}

if ( $pPort ne $lDefaultPort ) {
  $lConnectionMethod = 'port';
}

if ( $pHelp ) {
    &Usage();
    exit(0);
}

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort, Socket = $pSocket");
  exit($lError{'Unknown'});
}

if ( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my ($dbh, $sql, $sth);

if ( $lConnectionMethod eq 'socket' ) {

  # print "Connecting through socket.\n";
  $dbh = DBI->connect("DBI:mysql::mysql_socket=$pSocket"
                    , $pUser, $pPassword
                    , { RaiseError => 0
                      , PrintError => 0
                      , AutoCommit => 0
                      }
                    );
}
else {

  # print "Connecting through port.\n";
  $dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                    , $pUser, $pPassword
                    , { RaiseError => 0
                      , PrintError => 0
                      , AutoCommit => 0
                      }
                    );
}

if ( DBI::err ) {

  if ( DBI::errstr =~ m/Can't connect to/ ) {
    print("Error during connection: " . DBI::errstr . "\n");
    exit($lError{'Critical'});
  }

  if ( DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database: " . DBI::errstr . "\n");
    exit($lError{'Warning'});
  }

  print("Error during connection: " . DBI::errstr . "\n");
  exit($lError{'Critical'});
}

$sql = "
  SHOW GLOBAL STATUS WHERE variable_name in ('wsrep_cluster_size', 'wsrep_cluster_status')
";

$sth = $dbh->prepare( $sql );
if ( DBI::err ) {
  print("Error in preparing: " . DBI::errstr . "\n");
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my ($lKey, $lValue);
$sth->bind_columns(undef, \$lKey, \$lValue);
if ( $sth->err ) {
  print("Error in binding: " . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my %aStatus;
while ( $sth->fetchrow_arrayref() ) {
  $aStatus{$lKey} = $lValue;
}
if ( $sth->err ) {
  print("Error in fetchting:" . $sth->err);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}
$sth->finish;

if ( $aStatus{'wsrep_cluster_status'} eq "Primary" ) {

  if ( $aStatus{'wsrep_cluster_size'} == $pNodes ) {
    print "OK wsrep_cluster_size: " . $aStatus{'wsrep_cluster_size'} . ", wsrep_cluster_status: " . $aStatus{'wsrep_cluster_status'} . "\n";
    $dbh->disconnect;
    exit($lError{'OK'});
  }
  else {
    print "Caution: wsrep_cluster_size: " . $aStatus{'wsrep_cluster_size'} . ", wsrep_cluster_status: " . $aStatus{'wsrep_cluster_status'} . "\n";
    $dbh->disconnect;
    exit($lError{'Warning'});
  }
}
else {
  print "Caution: wsrep_cluster_status: " . $aStatus{'wsrep_cluster_status'} . "\n";
  $dbh->disconnect;
  exit($lError{'Critical'});
}

print "Galera Cluster for MySQL seems OK...";
exit($lError{'OK'});
