#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp = 0;
my $pUser = 'root';
my $pPassword = '';
my $pHost = 'localhost';
my $pPort = '3306';
my $pWarning = 2;
my $pCritical = 1;
my $pExpectedSlaves = 1;

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags

DESCRIPTION

  Nagios plugin to count connected slaves on host...

FLAGS

  critical, c  Number of slaves that are expected to be available (default $pCritical).
  expected, e  Number of slaves expected on master (default $pExpectedSlaves)
  help, ?      Print this help.
  host, h      Host where database is located (default $pHost)
  password, p  Password of user $pUser (default >$pPassword<)
  port         Port where database listens to (default $pPort)
  user, u      User for connecting to the database (default $pUser)
  warning, w   Number of slaves that are expected to be available (default $pWarning).

PARAMETERS

  none

EOF
}

my $rc = GetOptions(
    'user|u=s' => \$pUser
  , 'password|p=s' => \$pPassword
  , 'host|h=s' => \$pHost
  , 'port=i' => \$pPort
  , 'warning|w=i' => \$pWarning
  , 'critical|c=i' => \$pCritical
  , 'expected|e=i' => \$pExpectedSlaves
  , 'help|?' => \$pHelp
);

if ( $pHelp ) {
    &Usage();
    exit(0);

}

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my ($dbh, $sql, $sth);

$dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                  , $pUser, $pPassword
                  , { RaiseError => 0
                    , PrintError => 0
                    , AutoCommit => 0
                    }
                   );

if ( DBI::err ) {

  if ( DBI::errstr =~ m/Can't connect to/ ) {
    print("Error during connection: " . DBI::errstr);
    exit($lError{'Critical'});
  }

  if ( DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database: " . DBI::errstr);
    exit($lError{'Warning'});
  }

  print("Error during connection: " . DBI::errstr);
  exit($lError{'Critical'});
}

$sql = "
  SHOW SLAVE HOSTS
";

$sth = $dbh->prepare( $sql );
if ( DBI::err ) {
  print("Error in preparing: " . DBI::errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my ($lServerId, $lHost, $lPort, $lRplRecoveryRank
  , $lMasterId);
$sth->bind_columns(undef, \$lServerId, \$lHost, \$lPort, \$lRplRecoveryRank
  , \$lMasterId);
if ( $sth->err ) {
  print("Error in binding:" . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my $lCnt = 0;
while ( $sth->fetchrow_arrayref() ) {
  $lCnt++;
};
if ( $sth->err ) {
  print("Error in fetchting:" . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->finish;
$dbh->disconnect;

if ( $lCnt < $pCritical ) {
  print("Only $lCnt slaves are connected to the master which is less than the allowed CRITICAL level of $pCritical");
  exit($lError{'Critical'});
}
elsif ( $lCnt < $pWarning ) {
  print("Only $lCnt slaves are connected to the master which is less than the allowed WARNING level of $pWarning");
  exit($lError{'Warning'});
}

print("$lCnt slaves are connected to the master which is more or equal than the allowed WARNING level of $pWarning");
exit($lError{'OK'});
