#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp = 0;
my $pUser = 'root';
my $pPassword = '';
my $pHost = 'localhost';
my $pPort = '3306';

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags

DESCRIPTION

  Nagios plugin to check a MySQL slave if it is set to read-only...

FLAGS

  help, ?      Print this help.
  host, h      Host where database is located (default $pHost)
  password, p  Password of user $pUser (default >$pPassword<)
  port         Port where database listens to (default $pPort)
  user, u      User for connecting to the database (default $pUser)

PARAMETERS

  none

EOF
}

my $rc = GetOptions( 'help|?' => \$pHelp
                   , 'user|u=s' => \$pUser
                   , 'host|h=s' => \$pHost
                   , 'password|p=s' => \$pPassword
                   , 'port=i' => \$pPort
                   );

if ( $pHelp ) {
    &Usage();
    exit(0);

}

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my ($dbh, $sql, $sth);

$dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                  , $pUser, $pPassword
                  , { RaiseError => 0
                    , PrintError => 0
                    , AutoCommit => 0
                    }
                   );

if ( DBI::err ) {

  if ( DBI::errstr =~ m/Can't connect to/ ) {
    print("Error during connection: " . DBI::errstr);
    exit($lError{'Critical'});
  }

  if ( DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database: " . DBI::errstr);
    exit($lError{'Warning'});
  }

  print("Error during connection: " . DBI::errstr);
  exit($lError{'Critical'});
}

$sql = "
  SHOW VARIABLES LIKE 'read_only'
";

$sth = $dbh->prepare( $sql );
if ( DBI::err ) {
  print("Error in preparing: " . DBI::errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my ($lName, $lValue);
$sth->bind_columns(undef, \$lName, \$lValue);
if ( $sth->err ) {
  print("Error in binding: " . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
if ( $sth->err ) {
  print("Error in fetchting:" . $sth->err);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->finish;
$dbh->disconnect;

if ( $lValue eq  'OFF' ) {
  print("Database is read_write but should be read_only");
  exit($lError{'Critical'});
}

print "Database is read_only which is fine";
exit($lError{'OK'});
