#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;
use File::Basename;

my $lMyName = basename($0);

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

my $pHelp = 0;
my $pUser = 'root';
my $pPassword = '';
my $pHost = 'localhost';
my $pPort = '3306';
my $pWarning = 60;
my $pCritical = 600;

sub Usage
{
  print <<EOF;

SYNOPSIS

  $lMyName flags

DESCRIPTION

  Nagios plugin to check how many seconds slave is behind master...

FLAGS

  help, ?      Print this help.
  host, h      Host where database is located (default $pHost)
  password, p  Password of user $pUser (default >$pPassword<)
  port         Port where database listens to (default $pPort)
  user, u      User for connecting to the database (default $pUser)
  critical|c   Seconds for critical level (default $pCritical)
  warning|w    Seconds for warning level (default $pWarning)

PARAMETERS

  none

EOF
}

my $rc = GetOptions(
    'user|u=s' => \$pUser
  , 'password|p=s' => \$pPassword
  , 'host|h=s' => \$pHost
  , 'port=i' => \$pPort
  , 'warning|w=i' => \$pWarning
  , 'critical|c=i' => \$pCritical
  , 'help|h' => \$pHelp
);

if ( $pHelp ) {
    &Usage();
    exit(0);
}

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my ($dbh, $sql, $sth);

$dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                  , $pUser, $pPassword
                  , { RaiseError => 0
                    , PrintError => 0
                    , AutoCommit => 0
                    }
                   );

if ( DBI::err ) {

  if ( DBI::errstr =~ m/Can't connect to/ ) {
    print("Error during connection: " . DBI::errstr);
    exit($lError{'Critical'});
  }

  if ( DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database: " . DBI::errstr);
    exit($lError{'Warning'});
  }

  print("Error during connection: " . DBI::errstr);
  exit($lError{'Critical'});
}

$sql = "
  SHOW SLAVE STATUS
";

$sth = $dbh->prepare( $sql );
if ( DBI::err ) {
  print("Error in preparing: " . DBI::errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing: " . $sth->errstr);
  $dbh->disconnect;
  exit($lError{'Critical'});
}

my ($lSlaveIoState, $lMasterHost, $lMasterUser, $lMasterPort
  , $lConnectRetry, $lMasterLogFile
  , $lReadMasterLogPos, $lRelayLogFile, $lRelayLogPos
  , $lRelayMasterLogFile, $lSlaveIORunning
  , $lSlaveSQLRunning, $lReplicateDoDB, $lReplicateIgnoreDB
  , $lReplicateDoTable, $lReplicateIgnoreTable
  , $lReplicateWildDoTable, $lReplicateWildIgnoreTable, $lLastErrno
  , $lLastError, $lSkipCounter
  , $lExecMasterLogPos, $lRelayLogSpace, $lUntilCondition
  , $lUntilLogFile, $lUntilLogPos, $lMasterSSLAllowed
  , $lMasterSSLCAFile, $lMasterSSLCAPath, $lMasterSSLCert
  , $lMasterSSLCipher, $lMasterSSLKey, $lSecondsBehindMaster);

$sth->bind_columns(undef, \$lSlaveIoState, \$lMasterHost, \$lMasterUser
                 , \$lMasterPort, \$lConnectRetry, \$lMasterLogFile
                 , \$lReadMasterLogPos, \$lRelayLogFile, \$lRelayLogPos
                 , \$lRelayMasterLogFile, \$lSlaveIORunning
                 , \$lSlaveSQLRunning, \$lReplicateDoDB, \$lReplicateIgnoreDB
                 , \$lReplicateDoTable, \$lReplicateIgnoreTable
                 , \$lReplicateWildDoTable, \$lReplicateWildIgnoreTable
                 , \$lLastErrno, \$lLastError, \$lSkipCounter
                 , \$lExecMasterLogPos, \$lRelayLogSpace, \$lUntilCondition
                 , \$lUntilLogFile, \$lUntilLogPos, \$lMasterSSLAllowed
                 , \$lMasterSSLCAFile, \$lMasterSSLCAPath, \$lMasterSSLCert
                 , \$lMasterSSLCipher, \$lMasterSSLKey, \$lSecondsBehindMaster);
if ( $sth->err ) {
  print("Error in binding: " . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
if ( $sth->err ) {
  print("Error in fetchting: " . $sth->errstr);
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->finish;
$dbh->disconnect;

if (! defined($lSecondsBehindMaster) ) {
  print("Seconds_Behind_Master seems to be NULL");
  exit($lError{'Warning'});
}

if ( $lSecondsBehindMaster >= $pCritical ) {
  print("Database is $lSecondsBehindMaster s behind master which is more than the allowed CRITICAL level of $pCritical s");
  exit($lError{'Critical'});
}
elsif ( $lSecondsBehindMaster >= $pWarning ) {
  print("Database is $lSecondsBehindMaster s behind master which is more than the allowed WARNING level of $pWarning s");
  exit($lError{'Warning'});
}

print("Database is fine with $lSecondsBehindMaster s behind master which is less than the WARNING level of $pWarning s");
exit($lError{'OK'});
